import discord
import re
from actions.core import ActionCore

class ActionAVC(ActionCore):

    def avc_parent_hkey(self, txt):
        return f"avc:m:{txt}"

    def avc_channel_hkey(self, txt):
        return f"avc:c:{txt}"

    async def avc_ismon(self, vc: discord.VoiceChannel):
        """
        If we have an entry for monitoring this VC, returns True
        False otherwise
        """
        phkey = self.avc_parent_hkey(vc.id)
        if await redis.hexists(phkey, category_key):
            return True
        return False

    async def avc_isman(self, vc: discord.VoiceChannel):
        """
        If we have an entry for monitoring this VC, returns True
        False otherwise
        """
        chkey = self.avc_channel_hkey(vc.id)
        if await redis.hexists(chkey, category_key):
            return True
        return False

    async def avc_return_parent_category(self, vc: discord.VoiceChannel):
        """
        Accepts a discord.VoiceChannel as an input
        Returns either a custom set category or the VoiceChannels category

        Simplifies logic later when we try to determine "has the user configured a category for
        newly created voice channels"
        """
        # Set vars
        phkey        = self.avc_parent_hkey(vc.id)
        category_key = "category"

        if await self.redis.hexists(phkey, category_key):
            category_id = await self.redis.hget(phkey, category_key)
            category    = await self.bot.get_channel(category_id)
        else:
            category    = vc.category

        return category

        


    async def avc_create(self, member: discord.Member, before: discord.VoiceState, after: discord.VoiceState):
        """
        Checks to see if we're monitoring this channel.

        If we are, it'll check to see where it should create a new channel.
        Create the new channel. Move the user to it. Internally ID them as psuedo-owner.

        If it succeeds then it returns the new VC else it will return None
        """

        # Make sure we're monitoring
        if await self.avc_ismon(after):
            # If we are, grab it's category
            category = await self.avc_return_parent_category(after)
            
            # Configure the channel name for the vc
            channel_name = re.sub(r'[^a-z0-9]', '', member.nick.lower())
            channel_name = channel_name if channel_name else "someone"
            channel_name = channel_name.capitalize()
            channel_name = f"{channel_name}'s VC"

            # Create the voice channel
            new_vc = await category.create_voice_channel(channel_name)

            # Move the user to the voice channel
            await member.move_to(new_vc)

            # Register this new channel in REDIS
            chkey = self.avc_channel_hkey(new_vc.id)
            self.redis.hset(chkey, "guild", new_vc.guild.id)
            self.redis.hset(chkey, "owner", member.id)

            return new_vc

        return None


    async def avc_cleanup(self):
        """
        Runs through the REDIS server to find voice channels to delete.

        Should be run frequently.

        Can't imagine it needs a return for now.
        """

        async for key in self.redis.scan_iter(match="avc:c:*"):
            vc_id    = key.split(":")[2]
            guild_id = self.redis.hget(key, "guild")
            vc       = self.bot.channel_by_id(guild_id, vc_id)

            if len(vc.members) == 0:
                vc.delete(reason="Empty. AVC Cleanup")
                self.redis.hdel(key)


    async def avc_monitor(self, guild_id: int, channel_id: int):
        # Have to do this for redis...
        await self.setup()

        # Setup vars we need
        channel = await self.channel_by_id(guild_id, channel_id)
        phkey   = self.avc_parent_hkey(channel_id)
    
        # Check if we're about to monitor a VC just incase.
        # Cog should check this anyway.
        if not isinstance(channel, discord.VoiceChannel):
            return False
    
        # hset returns True if field is new or changed, else False
        result = await self.redis.hset(phkey, "guild", guild_id)
        if result:
            return True
        else:
            return False
