# cogs/admin.py

# Helpers for intent matching
from helpers.intent import intent
from helpers.intent_cog import IntentCog

# RE for stripping phrasing out
import re

class AdminCog(IntentCog):
    def __init__(self, bot):
        self.bot = bot

    # ------------
    # Help Message
    # ------------

    @intent([["admin"], ["help"]])
    async def guild_help(self, ctx, doc):
        await ctx.send("Full help is not really possible at this stage. I'll try to understand what you're saying and match it to what you're wanting to do.")
        lines = []
        for i, intent in enumerate(list_intents(), start=1):
            pattern_str = " + ".join(f"[{', '.join(group)}]" for group in intent["pattern"])
            lines.append(f"{i}. `{intent['func_name']}` from **{intent['cog']}** → {pattern_str}")

        message = "\n".join(lines) or "No intents registered."
        await ctx.send(f"**Registered Intents:**\n{message}")

    # ------------

    @intent([["kick", "remove"], ["user", "member"]])
    async def user_kick(self, ctx, doc):
        # Expected phrase to match
        # kick user @mention
        if ctx.guild == None:
            await ctx.send("I can't kick users from DMs")
            return

        targets = [m for m in ctx.message.mentions if m.id != ctx.bot.user.id]
        if targets:
            for target in targets:
                await target.kick()
            await ctx.send(f"I've kicked: {' '.join(target.mention for target in targets)}")
        else:
            await ctx.send("You need to @mention someone to kick them.")

    @intent([["ban"], ["user", "member"]])
    async def user_ban(self, ctx, doc):
        # Expected phrase to match
        # ban user @mention
        if ctx.guild == None:
            await ctx.send("I can't ban users from DMs")
            return

        targets = [m for m in ctx.message.mentions if m.id != ctx.bot.user.id]
        if targets:
            for target in targets:
                await target.ban()
            await ctx.send(f"I've banned: {' '.join(target.mention for target in targets)}")
        else:
            await ctx.send("You need to @mention someone to ban them.")

    @intent([["unban"], ["user", "member"]])
    async def user_unban(self, ctx, doc):
        output = []
    
        if ctx.guild is None:
            await ctx.send("I can't unban users from DMs. You have to do it within your guild, preferably in your staff chat.")
            return
    
        user_ids = re.findall(r'\b\d+\b', ctx.message.content)
        if not user_ids:
            output.append("I couldn't find any user IDs in your message.")
        else:
            banned_users = {}
            async for ban_entry in ctx.guild.bans():
                banned_users[ban_entry.user.id] = ban_entry.user
    
            for user_id in user_ids:
                user_id = int(user_id)
                if user_id not in banned_users:
                    output.append(f"User ID {user_id} is not currently banned.")
                    continue
    
                user = banned_users[user_id]
                try:
                    await ctx.guild.unban(user)
                    output.append(f"Unbanned user: {user.name}")
                except Exception as e:
                    output.append(f"Failed to unban user {user_id}: {e}")
    
        await ctx.send("\n".join(output))


    @intent([["give", "grant"], ["role"]])
    async def user_give_role(self, ctx, doc):
        # Expected phrase to match:
        # give role 123 to user @mention
        output = []

        if ctx.guild == None:
            output.append("There aren't any roles in DMs")
            return

        # Extracts a single role ID succeeding phrase "id [0-9]+"
        role_match = re.search(r'\b(?:id\s+)?(\d{3,})\b', ctx.message.content)
        role_id    = int(role_match.group(1)) if role_match else None

        # Extracts the mention of a single user following the phrase "user @user"
        user_match = re.search(r'\bto\s+(?:user\s+)?(<@!?\d+>)', ctx.message.content)
        user_id    = user_match.group(1) if user_match else None

        if not role_id:
            output.append("Could not find a role ID in your message")
        else:
            output.append(f"Role detected: {role_id}")

        if not user_id:
            output.append("Could not find a user mention in your message")
        else:
            output.append(f"User detected: {user_id}")

        output.append("Command completed")

        await ctx.send(output)




async def setup(bot):
    await bot.add_cog(AdminCog(bot))

