# Helpers for NLP intent declaration
from helpers.intent import intent, list_intents
from helpers.intent_cog import IntentCog

# Helper for logging
import helpers.logger as log_helper

class ControlCog(IntentCog):
    def __init__(self, bot):
        self.bot = bot
        self.OWNERS = [214577944570494976, 1088598275562668195]
        self.logger = log_helper.get_logger(__name__)

    @intent([["cog"], ["reload"]])
    async def cog_reload_intent(self, ctx, doc):
        # Set UUID for this operation
        UUID = log_helper.get_uuid()

        self.logger.debug(f"[{UUID}] Cog load called. Command: {doc}")

        if ctx.author.id not in self.OWNERS:
            await ctx.reply('Sorry but you are not authorized')
            return

        words = ctx.message.content.split()
        cog_name = " ".join(words[2:]).strip()
        cog_path = f"cogs.{cog_name}"

        if cog_path in self.bot.extensions:
            try:
                await self.bot.reload_extension(cog_path)
                await ctx.reply(f"Reloaded {cog_name}")
            except Exception as e:
                await ctx.reply(f"Error reloading {cog_name}: {e}")
        else:
            await ctx.reply(f"Cog {cog_name} is not loaded")

    @intent([["cog"], ["load"]])
    async def cog_load_intent(self, ctx, doc):
        # Set UUID for this operation
        UUID = log_helper.get_uuid()

        self.logger.debug(f"[{UUID}] Cog load called. Command: {doc}")

        if ctx.author.id not in self.OWNERS:
            await ctx.reply('Sorry but you are not authorized')
            return

        words = ctx.message.content.split()
        cog_name = " ".join(words[2:]).strip()
        cog_path = f"cogs.{cog_name}"

        if cog_path not in self.bot.extensions:
            try:
                await self.bot.load_extension(cog_path)
                await ctx.reply(f"Loaded {cog_name}")
            except Exception as e:
                await ctx.reply(f"Error loading {cog_name}: {e}")
        else:
            await ctx.reply(f"Cog {cog_name} is already loaded")

    @intent([["cog"], ["unload"]])
    async def cog_unload_intent(self, ctx, doc):
        # Set UUID for this operation
        UUID = log_helper.get_uuid()

        self.logger.debug(f"[{UUID}] Cog load called. Command: {doc}")

        if ctx.author.id not in self.OWNERS:
            await ctx.reply('Sorry but you are not authorized')
            return

        words = ctx.message.content.split()
        cog_name = " ".join(words[2:]).strip()
        cog_path = f"cogs.{cog_name}"

        if cog_path in self.bot.extensions:
            try:
                await self.bot.unload_extension(cog_path)
                await ctx.reply(f"Unloaded {cog_name}")
            except Exception as e:
                await ctx.reply(f"Error unloading {cog_name}: {e}")
        else:
            await ctx.reply(f"Cog {cog_name} is not loaded")

async def setup(bot):
    await bot.add_cog(ControlCog(bot))

